﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Drawing;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Common.Resources;
using HeuristicLab.Core.Networks;
using HeuristicLab.Core.Views;
using HeuristicLab.Networks.Programmable;
using HeuristicLab.PluginInfrastructure;
using HeuristicLab.Visualization;

namespace HeuristicLab.Networks.Views.NetworkVisualization.Views {
  public class AddPortsChartMode : ChartMode {
    protected TypeSelectorDialog typeSelectorDialog;

    public override Image Image { get { return VSImageLibrary.Interface; } }
    public override string ToolTip { get { return "Add Ports"; } }

    public AddPortsChartMode(ChartControl control) : base(control) { }

    public override void HandleOnMouseMove(object sender, MouseEventArgs e) {
      base.HandleOnMouseMove(sender, e);

      if (!(chartControl.Tag is IUserDefinedNetwork)) return;

      try {
        chartControl.PictureBox.Refresh();

        var nodeRectangles = chartControl.Chart.GetAllPrimitives(e.Location).OfType<NodeRectangle>();
        var nodeRectangle = nodeRectangles.FirstOrDefault();
        if (nodeRectangle == null) return;

        var size = chartControl.Chart.TransformWorldToPixel(nodeRectangle.Size);
        var ll = chartControl.Chart.TransformWorldToPixel(nodeRectangle.LowerLeft);
        var ur = chartControl.Chart.TransformWorldToPixel(nodeRectangle.UpperRight);

        using (var graphics = chartControl.PictureBox.CreateGraphics())
        using (var pen = new Pen(Color.Green) { Width = 2f }) {
          graphics.DrawRectangle(pen,
            ll.X,
            ur.Y,
            size.Width,
            size.Height);
        }
      } finally {
        chartControl.PictureBox.Update();
      }
    }

    public override void HandleOnMouseUp(object sender, MouseEventArgs e) {
      base.HandleOnMouseUp(sender, e);

      switch (e.Button) {
        case MouseButtons.Left:
          var network = chartControl.Tag as IUserDefinedNetwork;
          if (network == null) return;

          var nodeRectangles = chartControl.Chart.GetAllPrimitives(e.Location).OfType<NodeRectangle>();
          var nodeRectangle = nodeRectangles.FirstOrDefault();
          if (nodeRectangle == null) return;

          var node = nodeRectangle.NetworkItem;
          var port = CreatePort(node);

          var algNode = node as IAlgorithmNode;
          if (algNode != null) {
            algNode.Ports.Add(port);
            return;
          }

          var userDefNode = node as IUserDefinedNode;
          if (userDefNode != null) {
            userDefNode.Ports.Add(port);
            return;
          }
          break;
      }
    }

    #region Helpers
    protected virtual IPort CreatePort(INode node) {
      if (typeSelectorDialog == null) {
        typeSelectorDialog = new TypeSelectorDialog();
        typeSelectorDialog.Caption = "Select Port";
        typeSelectorDialog.TypeSelector.Caption = "Available Ports";
        typeSelectorDialog.TypeSelector.Configure(typeof(IPort), false, true);
      }

      if (typeSelectorDialog.ShowDialog(chartControl) == DialogResult.OK) {
        try {
          var n = (IPort)typeSelectorDialog.TypeSelector.CreateInstanceOfSelectedType();
          n.Name = GetUniqueName(n.Name, node);
          return n;
        } catch (Exception ex) {
          ErrorHandling.ShowErrorDialog(chartControl, ex);
        }
      }
      return null;
    }

    protected virtual string GetUniqueName(string originalName, INode node) {
      if (!node.Ports.ContainsKey(originalName))
        return originalName;

      string name;
      int index = 0;
      do {
        index++;
        name = originalName + index;
      } while (node.Ports.ContainsKey(name));
      return name;
    }
    #endregion
  }
}
