﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.Collections;
using HeuristicLab.Core;
using HeuristicLab.Core.Networks;
using HeuristicLab.Core.Networks.Views;
using HeuristicLab.MainForm;
using HeuristicLab.Networks.Views.NetworkVisualization.Views.ChartModes;
using HeuristicLab.Visualization;

namespace HeuristicLab.Networks.Views.NetworkVisualization.Views {
  [View("Network Visualization View")]
  [Content(typeof(Network), false)]
  [Content(typeof(INetwork), false)]
  public partial class NetworkVisualizationView : NetworkItemView {
    public new INetwork Content {
      get { return (INetwork)base.Content; }
      set { base.Content = value; }
    }

    public NetworkVisualizationView() {
      InitializeComponent();

      var initialMode = new NetworkSelectChartMode(chartControl);
      chartControl.AddChartModes(
        initialMode,
        new MoveChartMode(chartControl),
        new ZoomInChartMode(chartControl),
        new ZoomOutChartMode(chartControl),
        new AddNodesChartMode(chartControl),
        new AddPortsChartMode(chartControl),
        new ConnectPortsChartMode(chartControl)
      );
      chartControl.Mode = initialMode;
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      UpdateContent();
      chartControl.Tag = Content;
    }

    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      Content.Nodes.ItemsAdded += Nodes_ItemsAdded;
      Content.Nodes.ItemsRemoved += Nodes_ItemsRemoved;
      Content.Nodes.ItemsReplaced += Nodes_ItemsReplaced;
      Content.Ports.ItemsAdded += Ports_ItemsAdded;
      Content.Ports.ItemsRemoved += Ports_ItemsRemoved;
      Content.Ports.ItemsReplaced += Ports_ItemsReplaced;
    }

    protected override void DeregisterContentEvents() {
      Content.Nodes.ItemsAdded -= Nodes_ItemsAdded;
      Content.Nodes.ItemsRemoved -= Nodes_ItemsRemoved;
      Content.Nodes.ItemsReplaced -= Nodes_ItemsReplaced;
      Content.Ports.ItemsAdded -= Ports_ItemsAdded;
      Content.Ports.ItemsRemoved -= Ports_ItemsRemoved;
      Content.Ports.ItemsReplaced -= Ports_ItemsReplaced;
      base.DeregisterContentEvents();
    }

    private void UpdateContent() {
      if (Content == null) return;

      chartControl.Chart.Group.Clear();
      DrawNodes(Content.Nodes);
      ConnectNodes(Content.Nodes);
    }

    private void DrawNodes(IKeyedItemCollection<string, INode> nodes) {
      var chart = chartControl.Chart;

      foreach (var node in nodes.OfType<IVisualizableNode>()) {
        var nodeShape = new NodeRectangle(chart, node);
        chart.Group.Add(nodeShape);
      }
    }

    private void ConnectNodes(IKeyedItemCollection<string, INode> nodes) {
      var chart = chartControl.Chart;
      var nodeRectangles = chart.Group.Primitives.OfType<NodeRectangle>();
      var node2NodeShape = nodeRectangles.ToDictionary(k => k.NetworkItem.Name, v => v);

      foreach (var node in nodes) {
        var nodeShape = node2NodeShape[node.Name];
        foreach (var port in node.Ports.OfType<IConnectablePort>()) {
          var connectedPort = port.ConnectedPort as IConnectablePort;
          if (connectedPort == null) continue;

          var connectedNode = connectedPort.Parent;
          if (connectedNode == null) continue;

          var connectedShape = node2NodeShape[connectedNode.Name];
          var leftPortRectangle = nodeShape.GetPortRectangle(port);
          var rightPortRectangle = connectedShape.GetPortRectangle(connectedPort);

          var connectionShape = new ConnectionLine(chart, leftPortRectangle, rightPortRectangle);

          chart.Group.Add(connectionShape);
        }
      }
    }

    #region Event Handlers
    #region Nodes Event Handlers
    private void Nodes_ItemsAdded(object sender, CollectionItemsChangedEventArgs<INode> e) {
      // TODO: don't update everything
      UpdateContent();
    }

    private void Nodes_ItemsRemoved(object sender, CollectionItemsChangedEventArgs<INode> e) {
      // TODO: don't update everything
      UpdateContent();
    }

    private void Nodes_ItemsReplaced(object sender, CollectionItemsChangedEventArgs<INode> e) {
      // TODO: don't update everything
      UpdateContent();
    }
    #endregion

    #region Ports Event Handlers
    private void Ports_ItemsAdded(object sender, CollectionItemsChangedEventArgs<IPort> e) { }

    private void Ports_ItemsRemoved(object sender, CollectionItemsChangedEventArgs<IPort> e) { }

    private void Ports_ItemsReplaced(object sender, CollectionItemsChangedEventArgs<IPort> e) { }
    #endregion
    #endregion
  }
}
